#!/bin/sh
#   Copyright (c) 2010, 2012 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

#   Note: when called with "upgrade", "addon-upgrade" or "addon-install"
#   parameter, the "update" command is always executed first.

. /www/pages/cgi-bin/errlib.sh
. /www/pages/cgi-bin/util.sh

export PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin
export SHELL=/bin/sh

LOCKFILE="/var/lock/systemupdate"
REBOOT_TRIGGER="/var/lock/rebootafterupdate"
LOGDIR="/var/log/dss11-websetup"
mkdir -p $LOGDIR
UPDATE_CODE="$LOGDIR/last-update-code.txt"
UPDATE_MESSAGE="$LOGDIR/last-update-msg.txt"

UPDATE_CODE_DSM="$LOGDIR/last-update-code-dsmfw.txt"
UPDATE_MESSAGE_DSM="$LOGDIR/last-update-msg-dsmfw.txt"

UPDATE_CODE_ADDON="$LOGDIR/last-update-code-addon.txt"
UPDATE_MESSAGE_ADDON="$LOGDIR/last-update-msg-addon.txt"

UPDATE_DL_DIR="/var/lib/opkg/download"

REBOOT_AFTER_UPDATE=1

DSMFW_RETCODE=0
DSMS_NEED_UPDATE=0
FIRMWARE_DIRECTORY="/usr/share/dsm-firmware"
FW_TOOL="/usr/bin/dsm-firmware-upgrade"

signal_led_update() {
    if [[ ! -x /usr/bin/dss11-ledd-client ]]; then
        return 1
    fi

    if [[ "$1" == "start" ]]; then
        /usr/bin/dss11-ledd-client system_update_started
    elif [[ "$1" == "end" ]]; then
        /usr/bin/dss11-ledd-client system_update_completed
    fi
}

echo "[`date`] script doupdate.sh started" >> "$LOGDIR/update.log"

if [[ ! -x /usr/bin/opkg ]] || [[ ! -x /usr/bin/addon  ]]; then
    echo "opkg utility or addon wrapper not found"
    echo "[`date`] command \"$1\" end ---------------" >> "$LOGDIR/update.log"
    echo
    exit $ERR_SETUP
fi

if [[ -e "$LOCKFILE" ]]; then
    echo "Lock file exists, update is in progress..."
    echo "[`date`] command \"$1\" end ---------------"
    exit $ERR_DOWAIT
fi

OPKG_CMD=opkg
# support updates from USB
if [[ -x /usr/bin/opkg-usb ]] &&
   [[ -d /media/dss11usb/dss11-usb-upgrade ]]; then
    OPKG_CMD=opkg-usb
fi


if [[ "$1" == "addon-update" ]] || [[ "$1" == "addon-upgrade" ]] ||
   [[ "$1" == "addon-install" ]] || [[ "$1" == "addon-remove" ]]; then
    OPKG_CMD=addon
    UPDATE_CODE="$UPDATE_CODE_ADDON"
    UPDATE_MESSAGE="$UPDATE_MESSAGE_ADDON"
    if [[ -x /usr/bin/addon-usb ]] &&
       [[ -d /media/dss11usb/dss11-usb-upgrade ]]; then
        OPKG_CMD=addon-usb
    fi
fi

# we are good to go
touch "$LOCKFILE"

# reset messages
set_update_message
echo "$ERR_IGNORE" > $UPDATE_CODE

# clean temporary opkg directories
if [[ -d /var/lib/opkg/tmp ]]; then
    rm -rf /var/lib/opkg/tmp/*
fi

if [[ -d /var/lib/opkg-add-ons/tmp ]]; then
    rm -rf /var/lib/opkg-add-ons/tmp/*
fi

# run opkg update to refresh feeds for all commands except for "addon-remove"
if [[ "$1" != "addon-remove" ]]; then
    set_update_code $UPDATE_STATUS_FETCHING_PKGLIST

# UI has special handling for the addons, no detailed progress is required
# there, so do not overload the UI messages here
    COMMAND="$1"
    if [[ "${COMMAND:0:5}" != "addon" ]]; then
        set_update_message "Downloading package lists"
    fi
    echo "Executing: $OPKG_CMD update"
    $OPKG_CMD update
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Failed to refresh package information!" > $UPDATE_MESSAGE
        echo "Failed to refresh package information, opkg returned: $RETCODE"
        echo "[`date`] command \"$1\" end ---------------"
        echo
        set_update_message
        rm -f "$LOCKFILE"
        exit $ERR_OPKG
    fi
fi

if [[ "$1" == "upgrade" ]] || [[ "$1" == "addon-upgrade" ]]; then
    CMD="$1"
    if [[ "$1" == "addon-upgrade" ]]; then
        set_update_code $UPDATE_STATUS_INSTALLING
        set_update_message
        shift 1
        echo "Executing: $OPKG_CMD upgrade $@"
        $OPKG_CMD upgrade $@
        RETCODE="$?"
    else
        signal_led_update "start"
        if [[ -s /etc/dSS11-image-version ]]; then
            echo -n "Updating from image version: "
            cat /etc/dSS11-image-version
        fi

        if [[ -n "$UPDATE_DL_DIR" ]] && [[ -d "$UPDATE_DL_DIR" ]]; then
            echo "Cleaning download directory $UPDATE_DL_DIR/*"
            rm -f $UPDATE_DL_DIR/*
        else
            echo "Creating download directory $UPDATE_DL_DIR"
            mkdir -p "$UPDATE_DL_DIR"
            RETCODE="$?"
            if [[ "$RETCODE" -ne "0" ]]; then
                echo "Could not create download directory!"
                echo "$ERR_UPDATE" > $UPDATE_CODE
                echo "Could not create download directory" > $UPDATE_MESSAGE
                echo "[`date`] command \"$CMD\" end ---------------"
                echo
                signal_led_update "end"
                set_update_message
                rm -f "$LOCKFILE"
                exit $ERR_UPDATE
            fi
        fi

        REQ_SPACE=$(expr `opkg2json --upgrade-size | awk '{ print $9 }'` \* 2)
        AVAILABLE_SPACE=$(expr `df -k | grep -e " /$" | awk '{ print $(NF-2) }'` \* 1024)

        echo "Required space for update: $REQ_SPACE"
        echo "Available free space for update: $AVAILABLE_SPACE"

        if [[ -z "$REQ_SPACE" ]] || [[ -z "$AVAILABLE_SPACE" ]]; then
            echo "ERROR: Could not calculate free space needed for update, proceeding anyway!"
        else
            if [[ $AVAILABLE_SPACE -lt $REQ_SPACE ]]; then
                echo "Not enough free space for update!"
                echo "$ERR_UPDATE" > $UPDATE_CODE
                echo "Not enough free space for update!" > $UPDATE_MESSAGE
                echo "[`date`] command \"$CMD\" end ---------------"
                echo
                signal_led_update "end"
                set_update_message
                rm -f "$LOCKFILE"
                exit $ERR_UPDATE
            fi
        fi

        set_update_code $UPDATE_STATUS_DOWNLOADING
        set_update_message "Downloading packages, this may take a while depending on the speed of your internet connection. Do not power off and do not reboot the dSS!"
        cd "$UPDATE_DL_DIR"
        echo "Executing: $OPKG_CMD --download-only upgrade"
        $OPKG_CMD --download-only upgrade
        RETCODE="$?"
        if [[ "$RETCODE" -ne "0" ]]; then
            echo "Error when downloading packages, opkg returned: $RETCODE"
            echo "Cleaning download directory $UPDATE_DL_DIR"
            rm -f $UPDATE_DL_DIR/*
            echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
            echo "Failed to download packages, please try again later!" > $UPDATE_MESSAGE
            echo "[`date`] command \"$CMD\" end ---------------"
            echo
            signal_led_update "end"
            set_update_message
            rm -f "$LOCKFILE"
            exit $ERR_OPKG
        fi

        set_update_code $UPDATE_STATUS_INSTALLING
        set_update_message "Performing update, this may take up to 30 minutes. The dSS must not be powered off or rebooted during the update! While updating, operation within the rooms might be limited."
        echo "Executing: $OPKG_CMD upgrade --cache=$UPDATE_DL_DIR"
        $OPKG_CMD upgrade --cache=$UPDATE_DL_DIR
        RETCODE="$?"
        echo "Cleaning download directory $UPDATE_DL_DIR/*"
        rm -f $UPDATE_DL_DIR/*
        if [[ -s /etc/dSS11-image-version ]]; then
            echo -n "Update finished, new image version: "
            cat /etc/dSS11-image-version
        fi
    fi

    if [[ "$1" == "upgrade" ]]; then
        set_update_code $UPDATE_STATUS_DSM_FIRMWARE
        echo "Checking how many dSMs need upgrade"
        set_update_message "Checking if dSMs need upgrade..."
        ARMFW=`ls $FIRMWARE_DIRECTORY/dsm-arm*.img 2>/dev/null`
        DSPFW=`ls $FIRMWARE_DIRECTORY/dsm-dsp*.img 2>/dev/null`
        $FW_TOOL -p -B $ARMFW $DSPFW  > /dev/null 2>&1
        DSMS_NEED_UPDATE="$?"
        echo "dSM firmware check returned $DSMS_NEED_UPDATE"
        if [[ $DSMS_NEED_UPDATE -gt 128 ]]; then
            DSMFW_RETCODE=$ERR_UPDATE
            echo "$ERR_UPDATE:$DSMS_NEED_UPDATE" > $UPDATE_CODE_DSM
            echo "Error when checking dSM firmware" > $UPDATE_MESSAGE_DSM
            set_update_message "Error when checking dSM firmware"
        elif [[ $DSMS_NEED_UPDATE -gt 0 ]]; then
            echo "Calling dSM firmware upgrade script dsmfwupgrade.sh"
            sh /www/scripts/dsmfwupgrade.sh nolock
            DSMFW_RETCODE="$?"
            echo "dSM firmware upgrade script returned $DSMFW_RETCODE"
        elif [[ $DSMS_NEED_UPDATE -eq 0 ]]; then
            set_update_message "All dSMs are up to date"
            DSMFW_RETCODE=0
        fi
        # give UI time to poll the message
        usleep 5000000
    fi

    RET_ERR=$ERR_OK
    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Encountered errors during upgrade, please review the log!" > $UPDATE_MESSAGE
        echo "Encountered errors during upgrade, opkg returned: $RETCODE"
        RET_ERR=$ERR_UPDATE
    else
        SUBCODE_DSM=0 # unknown error
        if [[ -e $UPDATE_CODE_DSM ]]; then
            SUBCODE_DSM=`cat $UPDATE_CODE_DSM | cut -d ':' -f 2 2>/dev/null`
        fi
        MSG_DSM="OK"
        if [[ -e $UPDATE_MESSAGE_DSM ]]; then
            MSG_DSM=`cat $UPDATE_MESSAGE_DSM 2>/dev/null`
        fi

        if [[ "$DSMFW_RETCODE" -ne "0" ]]; then
            RET_ERR="$ERR_UPDATE"
            echo "$ERR_DSMFW:$SUBCODE_DSM" > "$UPDATE_CODE"
            echo "$MSG_DSM" > $UPDATE_MESSAGE
            set_update_message $MSG_DSM
        else
            if [[ $DSMS_NEED_UPDATE -gt $SUBCODE_DSM ]]; then
                echo "$ERR_DSMFW:139" > $UPDATE_CODE # 139 missed update
                echo "Not all dSMs have been upgraded" > $UPDATE_MESSAGE
                set_update_message "Not all dSMs have been upgraded"
            else
                echo "$ERR_OK" > $UPDATE_CODE
                echo "OK" > $UPDATE_MESSAGE
                set_update_message "dSM update finished"
            fi
        fi
    fi

    # some package requested a reboot after update
    if [[ -e $REBOOT_TRIGGER ]]; then
        echo "Requested reboot after update!"
        if [[ -x /sbin/shutdown ]]; then
            set_update_code $UPDATE_STATUS_REBOOTING
            set_update_message "The system needs to be restarted and will reboot automatically."
            # give the UI a change to pick up the status
            usleep 5000000
            echo "Rebooting system now"
            /sbin/shutdown -r now
            exit $ERR_OK
        else
            echo "Shutdown utility not found, rebooting is not possible!"
            rm -f "$REBOOT_TRIGGER"
            set_update_message
        fi
    fi

    if [[ "$1" == "upgrade" ]]; then
        if [[ $REBOOT_AFTER_UPDATE -eq 1 ]]; then
            GATEWAY=`route -n | grep UG | grep eth0 | awk '{ print $2}'`
            if [[ -n "$GATEWAY" ]] && [[ -x /bin/ping ]]; then
                echo "Checking network connection, will ping gateway $GATEWAY"
                /bin/ping -c 1 $GATEWAY
                if [[ "$?" -eq "0" ]]; then
                    echo "Network is OK, reboot not needed"
                    REBOOT_AFTER_UPDATE=0
                else
                    echo "No ping reply from $GATEWAY, will reboot"
                fi
            fi
        fi

        if [[ $REBOOT_AFTER_UPDATE -eq 1 ]]; then
            echo "Requested reboot after update!"
            if [[ -x /sbin/shutdown ]]; then
                set_update_code $UPDATE_STATUS_REBOOTING
                set_update_message "The system needs to be restarted and will reboot automatically."
                # give the UI a change to pick up the status
                usleep 5000000
                /sbin/shutdown -r now
                exit $ERR_OK
            else
                echo "Shutdown utility not found, rebooting is not possible!"
                rm -f "$REBOOT_TRIGGER"
                set_update_message
            fi
        fi
    fi
    signal_led_update "end"
    set_update_message
    rm -f "$LOCKFILE"
    echo "[`date`] command \"$CMD\" end ---------------"
    echo
    exit $RET_ERR
elif [[ "$1" == "addon-install" ]]; then
    set_update_code $UPDATE_STATUS_INSTALLING
    shift 1
    echo "Executing: $OPKG_CMD install $@"
    $OPKG_CMD install $@
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Encountered errors during addon install, please review the log!" > $UPDATE_MESSAGE
        echo "Encountered errors during addon install, opkg returned: $RETCODE"
        echo "[`date`] command \"addon-install\" end ---------------"
        echo
        set_update_message
        rm -f "$LOCKFILE"
        exit $ERR_UPDATE
    fi
    echo "[`date`] command \"addon-install\" end ---------------"
    echo
elif [[ "$1" == "addon-remove" ]]; then
    set_update_code $UPDATE_STATUS_REMOVING
    shift 1
    echo "Executing: $OPKG_CMD remove $@"
    $OPKG_CMD remove $@
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Encountered errors during addon removal, please review the log!" > $UPDATE_MESSAGE
        echo "Encountered errors during addon removal, opkg returned: $RETCODE"
        echo "[`date`] command \"addon-remove\" end ---------------"
        echo
        set_update_message
        rm -f "$LOCKFILE"
        exit $ERR_UPDATE
    fi
    echo "[`date`] command \"addon-remove\" end ---------------"
    echo

else
    echo "[`date`] command \"$1\" end ---------------"
    echo
fi

set_update_message
rm -f "$LOCKFILE"

echo "$ERR_OK" > $UPDATE_CODE
echo "OK" > $UPDATE_MESSAGE
exit $ERR_OK
